function toggle(s, t, c)
   return c(0, t, s)
end

function one(s, t, c)
   return c(1, s, t)
end

function height(s, t, c)
   return c(#s, s, t)
end

function void(n)
   return function (s, t, c)
      return n(s, t, function (x, y, z)
         return c(0, y, z)
      end)
   end
end

function neg(n)
   return function (s, t, c)
      return n(s, t, function (x, y, z)
         return c(-x, y, z)
      end)
   end
end

function pop(s, t, c)
   local u = {}
   for i = 2, #s do
      u[i-1] = s[i]
   end
   return c(s[1], u, t)
end

function loop(n)
   return function (s, t, c)
      if #s == 0 or s[1] == 0 then
         return c(0, s, t)
      else
         return concat(n, loop(n))(s, t, c)
      end
   end
end

function concat(m, n)
   return function (s, t, c)
      return m(s, t, function (x, y, z)
         return n(y, z, function (p, q, r)
            return c(x+p, q, r)
         end)
      end)
   end
end

function push(n)
   return function (s, t, c)
      return n(s, t, function (x, y, z)
         local u = {x}
         for i = 1, #y do
            u[i+1] = y[i]
         end
         return c(x, u, z)
      end)
   end
end

function parse(f, s, c)
   local nilads = { ['()']=one, ['[]']=height, ['{}']=pop, ['<>']=toggle }
   local monads = { ['(']=push, ['[']=neg, ['{']=loop, ['<']=void }
   local nilad = nilads[string.sub(s, 1, 2)]
   local monad = monads[string.sub(s, 1, 1)]
   if nilad then
      return parse(concat(f, nilad), string.sub(s, 3), c)
   elseif monad then
      return parse(void(one), string.sub(s, 2), function (g, s)
         return parse(concat(f, monad(g)), string.sub(s, 2), c)
      end)
   else
      return c(f, s)
   end
end

function brainflak(s, n)
   s = string.gsub(s, "[^%[%](){}<>]", "")
   s, t = parse(void(one), s, function (f, s) return f, s end)
   return s(n, {}, function (x, y, z) return y end)
end